
# ̓NX̃eXgP[X

require 'test/unit'
require 'string_parser'

class TC_StringParser < Test::Unit::TestCase
  def setup
  end

  def test_phrase
    str = "one"
    parser = StringParser.new( str )
    assert_equal("one", parser.search_phrase)
    assert_nil( parser.attribute_condition_ary )

    str = "one two"
    parser = StringParser.new( str )
    assert_equal("one AND two", parser.search_phrase)
    assert_nil( parser.attribute_condition_ary )
  
    str = "dog@cat"
    parser = StringParser.new( str )
    assert_equal("dog AND cat", parser.search_phrase)
    assert_nil( parser.attribute_condition_ary )
  
    str = "dog -cat"
    parser = StringParser.new( str )
    assert_equal("dog ANDNOT cat", parser.search_phrase)
    assert_nil( parser.attribute_condition_ary )
    
    str = "dog -cat  sheep"
    parser = StringParser.new( str )
    assert_equal("dog ANDNOT cat AND sheep", parser.search_phrase)
    assert_nil( parser.attribute_condition_ary )
    
    str = "black OR@ white"
    parser = StringParser.new( str )
    assert_equal("black OR white", parser.search_phrase)
    assert_nil( parser.attribute_condition_ary )
    
    str = "\"i love you\""
    parser = StringParser.new( str )
    assert_equal("i love you", parser.search_phrase)
    assert_nil( parser.attribute_condition_ary )

    str = "\"i love you\"  \"very much\""
    parser = StringParser.new( str )
    assert_equal("i love you AND very much", parser.search_phrase)
    assert_nil( parser.attribute_condition_ary )

    str = "\"very pretty\" dog  cat  "
    parser = StringParser.new( str )
    assert_equal("very pretty AND dog AND cat", parser.search_phrase)
    assert_nil( parser.attribute_condition_ary )

    # Ov
    str = "dog*"
    parser = StringParser.new( str )
    assert_equal("[BW] dog", parser.search_phrase)
    assert_nil( parser.attribute_condition_ary )

    # v
    str = "*cat"
    parser = StringParser.new( str )
    assert_equal("[EW] cat", parser.search_phrase)
    assert_nil( parser.attribute_condition_ary )
  end
  
  def test_attribute
    str = "hoge from:mike"
    parser = StringParser.new( str )
    assert_equal("hoge", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 1, parser.attribute_condition_ary.size )
    assert_equal( "from ISTRINC mike", parser.attribute_condition_ary[0] )
  
    str = "hoge  to:godzilla"
    parser = StringParser.new( str )
    assert_equal("hoge", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 1, parser.attribute_condition_ary.size )
    assert_equal( "to ISTRINC godzilla", parser.attribute_condition_ary[0] )

    str = "good  to,cc:member"
    parser = StringParser.new( str )
    assert_equal("good", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 1, parser.attribute_condition_ary.size )
    assert_equal( "to,cc ISTRINC member", parser.attribute_condition_ary[0] )

    str = "from:tom to,cc:yoshio"
    parser = StringParser.new( str )
    assert_equal("[UVSET]", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 2, parser.attribute_condition_ary.size )
    assert_equal( "from ISTRINC tom", parser.attribute_condition_ary[0] )
    assert_equal( "to,cc ISTRINC yoshio", parser.attribute_condition_ary[1] )

    # 2006/01/01  1970/01/01 00:00:00  1136041200 b
    str = "party after:2006/01/01"
    parser = StringParser.new( str )
    assert_equal("party", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 1, parser.attribute_condition_ary.size )
    assert_equal( "@cdate NUMGE 1136041200", parser.attribute_condition_ary[0] )

    # 2005/01/01  1970/01/01 00:00:00  1104505200 b
    # 2005/12/31  2005/12/31 23:59:59 ܂ł͔͈͓Ƃ̂ŁAu2006/01/01 00:00:00 菬vƂ 
    str = "YN after:2005/01/01 before:2005/12/31"
    parser = StringParser.new( str )
    assert_equal("YN", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 2, parser.attribute_condition_ary.size )
    assert_equal( "@cdate NUMGE 1104505200", parser.attribute_condition_ary[0] )
    assert_equal( "@cdate NUMLT 1136041200", parser.attribute_condition_ary[1] )
    
    # OR̃eXg
    str = "good to:member OR to:customer"
    parser = StringParser.new( str )
    assert_equal("good", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 1, parser.attribute_condition_ary.size )
    assert_equal( "to ISTRRX member|customer", parser.attribute_condition_ary[0] )

    str = "from:yamada OR from:tanaka"
    parser = StringParser.new( str )
    assert_equal("[UVSET]", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 1, parser.attribute_condition_ary.size )
    assert_equal( "from ISTRRX yamada|tanaka", parser.attribute_condition_ary[0] )

    # () ł̃O[ṽeXg
    str = " subject:([H f)"
    parser = StringParser.new( str )
    assert_equal("[UVSET]", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 2, parser.attribute_condition_ary.size )
    assert_equal( "subject ISTRINC [H", parser.attribute_condition_ary[0] )
    assert_equal( "subject ISTRINC f", parser.attribute_condition_ary[1] )

    str = " subject:([H OR f)"
    parser = StringParser.new( str )
    assert_equal("[UVSET]", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 1, parser.attribute_condition_ary.size )
    assert_equal( "subject ISTRRX [H|f", parser.attribute_condition_ary[0] )

    str = "ō \ A to,cc:(Rc OR c OR ē)"
    parser = StringParser.new( str )
    assert_equal("ō AND \ AND A", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 1, parser.attribute_condition_ary.size )
    assert_equal( "to,cc ISTRRX Rc|c|ē", parser.attribute_condition_ary[0] )

    # ɂẴt[Y
    str = "subject:\"Sunday service\""
    parser = StringParser.new( str )
    assert_equal("[UVSET]", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 1, parser.attribute_condition_ary.size )
    assert_equal( "subject ISTRINC Sunday service", parser.attribute_condition_ary[0] )
    
    # NOT
    str = "-from:yamada"
    parser = StringParser.new( str )
    assert_equal("[UVSET]", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 1, parser.attribute_condition_ary.size )
    assert_equal( "from !ISTRINC yamada", parser.attribute_condition_ary[0] )

    str = "from:-yamada"
    parser = StringParser.new( str )
    assert_equal("[UVSET]", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 1, parser.attribute_condition_ary.size )
    assert_equal( "from !ISTRINC yamada", parser.attribute_condition_ary[0] )

    str = "to:tanaka -from:yamada"
    parser = StringParser.new( str )
    assert_equal("[UVSET]", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 2, parser.attribute_condition_ary.size )
    assert_equal( "to ISTRINC tanaka", parser.attribute_condition_ary[0] )
    assert_equal( "from !ISTRINC yamada", parser.attribute_condition_ary[1] )

    str = "to,cc:tanaka -to,cc:yamada"
    parser = StringParser.new( str )
    assert_equal("[UVSET]", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 2, parser.attribute_condition_ary.size )
    assert_equal( "to,cc ISTRINC tanaka", parser.attribute_condition_ary[0] )
    assert_equal( "to,cc !ISTRINC yamada", parser.attribute_condition_ary[1] )

    str = "-to,cc:tanaka to,cc:yamada"
    parser = StringParser.new( str )
    assert_equal("[UVSET]", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 2, parser.attribute_condition_ary.size )
    assert_equal( "to,cc !ISTRINC tanaka", parser.attribute_condition_ary[0] )
    assert_equal( "to,cc ISTRINC yamada", parser.attribute_condition_ary[1] )

    str = "-to,cc:tanaka -to,cc:yamada"
    parser = StringParser.new( str )
    assert_equal("[UVSET]", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 2, parser.attribute_condition_ary.size )
    assert_equal( "to,cc !ISTRINC tanaka", parser.attribute_condition_ary[0] )
    assert_equal( "to,cc !ISTRINC yamada", parser.attribute_condition_ary[1] )

    str = " subject:(one two -three)"
    parser = StringParser.new( str )
    assert_equal("[UVSET]", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 3, parser.attribute_condition_ary.size )
    assert_equal( "subject ISTRINC one", parser.attribute_condition_ary[0] )
    assert_equal( "subject ISTRINC two", parser.attribute_condition_ary[1] )
    assert_equal( "subject !ISTRINC three", parser.attribute_condition_ary[2] )

    str = "from:(yama  da)"
    parser = StringParser.new( str )
    assert_equal("[UVSET]", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 2, parser.attribute_condition_ary.size )
    assert_equal( "from ISTRINC yama", parser.attribute_condition_ary[0] )
    assert_equal( "from ISTRINC da", parser.attribute_condition_ary[1] )

    str = " subject:(one -two -three)"
    parser = StringParser.new( str )
    assert_equal("[UVSET]", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 3, parser.attribute_condition_ary.size )
    assert_equal( "subject ISTRINC one", parser.attribute_condition_ary[0] )
    assert_equal( "subject !ISTRINC two", parser.attribute_condition_ary[1] )
    assert_equal( "subject !ISTRINC three", parser.attribute_condition_ary[2] )

    str = "subject:(YN OR VN -2007)"
    parser = StringParser.new( str )
    assert_equal("[UVSET]", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 2, parser.attribute_condition_ary.size )
    assert_equal( "subject ISTRRX YN|VN", parser.attribute_condition_ary[0] )
    assert_equal( "subject !ISTRINC 2007", parser.attribute_condition_ary[1] )

    str = "subject:(YN OR VN -2007) to,cc:yamada -from:takahashi \@s"
    parser = StringParser.new( str )
    assert_equal("\ AND s", parser.search_phrase)
    assert_not_nil( parser.attribute_condition_ary )
    assert_equal( 4, parser.attribute_condition_ary.size )
    assert_equal( "subject ISTRRX YN|VN", parser.attribute_condition_ary[0] )
    assert_equal( "subject !ISTRINC 2007", parser.attribute_condition_ary[1] )
    assert_equal( "to,cc ISTRINC yamada", parser.attribute_condition_ary[2] )
    assert_equal( "from !ISTRINC takahashi", parser.attribute_condition_ary[3] )
  end
end
